/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.model.models.cotisation;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import fr.amapj.model.engine.Identifiable;
import fr.amapj.model.models.editionspe.EditionSpecifique;

@Entity
@Table( uniqueConstraints=
{
   @UniqueConstraint(columnNames={"nom"})
})
public class PeriodeCotisation implements Identifiable
{

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long id;

	@NotNull
	@Size(min = 1, max = 100)
	@Column(length = 100)
	public String nom;
	
	@NotNull
	// Montant mini 
	public int montantMini;
	
	@NotNull
	// Montant conseillé
	public int montantConseille;
	
	@NotNull
	@Temporal(TemporalType.DATE)
	public Date dateDebut;
	
	@NotNull
	@Temporal(TemporalType.DATE)
	public Date dateFin;
	
	
	// Paiement
	
	@Size(min = 0, max = 2048)
	@Column(length = 2048)
	// Texte qui sera affiché pour le paiement
	public String textPaiement;
	
	// Libellé du chéque 
	@Size(min = 0, max = 100)
	@Column(length = 100)
	public String libCheque;
	
	
	/**
	 * Bulletin d'adhesion au format PDF
	 * null si il n'y a pas de bulletin d'adhesion   
	 */
	@ManyToOne
	public EditionSpecifique bulletinAdhesion;

	public Long getId()
	{
		return id;
	}

	public void setId(Long id)
	{
		this.id = id;
	}

}
