/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.service.engine.objectstorage;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import fr.amapj.common.Base64Utils;
import fr.amapj.common.GzipUtils;
import fr.amapj.common.SQLUtils;
import fr.amapj.model.models.objectstorage.ObjectStorage;

public class LocalObjectStorageService implements ObjectStorageService
{
	
	LocalObjectStorageService()
	{
		
	}
	
	/**
	 * 
	 * content doit être non null
	 */
	public void updateOrCreateObject(EntityManager em,String key,byte[] content,boolean compress)
	{
		ObjectStorage os = findObjectSorage(em, key);
		
		if (os==null)
		{
			os = new ObjectStorage();
			os.key = key;
			em.persist(os);
		}
		os.content = compress ? GzipUtils.compressByteArray(content) : Base64Utils.toBase64(content);
	}
		
	
	/**
	 * Retourne null si l'objet n'existe pas 
	 */
	public byte[] loadObject(EntityManager em,String key,boolean uncompress)
	{
		ObjectStorage os = findObjectSorage(em, key);
		if (os==null)
		{
			return null;
		}
		return uncompress ? GzipUtils.uncompressByteArray(os.content) : Base64Utils.fromBase64(os.content);
	}

	public void deleteObject(EntityManager em, String key)
	{
		ObjectStorage os = findObjectSorage(em, key);
		em.remove(os);	
	}
	
	private ObjectStorage findObjectSorage(EntityManager em,String key)
	{
		TypedQuery<ObjectStorage> q = em.createQuery("select p from ObjectStorage p where p.key=:key ",ObjectStorage.class);
		q.setParameter("key", key);
		return SQLUtils.oneOrZero(q); 
	}
}
